/**
 * Required External Modules and Interfaces
 */

import express, {Request, Response} from 'express';
import {Event} from './event.interface';
import * as EventService from './events.service';
import * as iCalService from './icalgenerator.service';
import * as CredentialService from './credentials.service';
import * as UserService from '../users/users.service';
import {Guid} from 'guid-typescript';
import logger from '../../../middleware/logger';


/**
 * Router Definition
 */

export const eventsRouter = express.Router();

/**
 * Constants
 */
export const calendarNames = new Map<string, any>([
	['public', {id: 1, name: 'Nachklang_calendar'}],
	['members', {id: 2, name: 'Nachklang_internal_calendar'}],
	['choir', {id: 4, name: 'Nachklang_choir_calendar'}],
	['management', {id: 3, name: 'Nachklang_management_calendar'}]
]);


/**
 * Controller Definitions
 */

/**
 * @swagger
 * /calendar/events/{calendar}/json:
 *      get:
 *          summary: Get all events from a specific calendar in JSON format
 *          description: Returns all events from the specified calendar in JSON format. Authentication required.
 *          tags:
 *              - calendar
 *          parameters:
 *              - in: path
 *                name: calendar
 *                required: true
 *                schema:
 *                    type: string
 *                    enum: [public, members, choir, management]
 *                description: The name of the calendar to get events from
 *              - in: query
 *                name: sessionId
 *                schema:
 *                    type: string
 *                description: Session ID for authentication
 *              - in: query
 *                name: sessionKey
 *                schema:
 *                    type: string
 *                description: Session key for authentication
 *              - in: query
 *                name: password
 *                schema:
 *                    type: string
 *                description: Password for calendar access (if not using session authentication)
 *          responses:
 *              200:
 *                  description: Success
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: array
 *                              items:
 *                                  $ref: '#/components/schemas/Event'
 *              400:
 *                  description: Bad request - missing or invalid parameters
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: Please state the name of the calendar you want events from.
 *              403:
 *                  description: Forbidden - no access to the calendar
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: You do not have access to the specified calendar.
 *              500:
 *                  description: Server error
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: Internal Server Error. Try again later.
 */
eventsRouter.get('/:calendar/json', async (req: Request, res: Response) => {
	try {
		// Get request params
		let calendarName: string = req.params.calendar as string ?? '';
		let sessionId: string = req.query.sessionId as string ?? '';
		let sessionKey: string = req.query.sessionKey as string ?? '';
		let password: string = req.query.password as string ?? '';
		let ip: string = req.socket.remoteAddress ?? '';

		if (calendarName.length < 1) {
			res.status(400).send({'message': 'Please state the name of the calendar you want events from.'});
			return;
		}

		if (!Array.from(calendarNames.keys()).includes(calendarName)) {
			res.status(400).send({'message': 'Unknown calendar.'});
			return;
		}

		let calendarId: number = calendarNames.get(calendarName)!.id;

		let user = await UserService.checkSession(sessionId, sessionKey, ip);

		// If no user was found, check if the password gives access to the calendar
		if(user === null || !user.isActive) {
			if (! await CredentialService.hasAccess(calendarName, sessionId, sessionKey, password, ip)) {
				res.status(403).send({'message': 'You do not have access to the specified calendar.'});
				return;
			}
		}

		let events: Event[];

		if(user.isActive) {
			events = await EventService.getAllEventsAdmin(calendarId);
		} else {
			events = await EventService.getAllEvents(calendarId);
		}

		// Send the events back
		res.status(200).send(events);
	} catch (e: any) {
		console.log('Error handling a request: ' + e.message);
		res.status(500).send({'message': 'Internal Server Error. Try again later.'});
	}
});

/**
 * @swagger
 * /calendar/events/{calendar}/json/next:
 *      get:
 *          summary: Get the next upcoming event from a calendar
 *          description: Returns the next upcoming event from the specified calendar. Authentication required.
 *          tags:
 *              - calendar
 *          parameters:
 *              - in: path
 *                name: calendar
 *                required: true
 *                schema:
 *                    type: string
 *                    enum: [public, members, choir, management]
 *                description: The name of the calendar to get the next event from
 *              - in: query
 *                name: sessionId
 *                schema:
 *                    type: string
 *                description: Session ID for authentication
 *              - in: query
 *                name: sessionKey
 *                schema:
 *                    type: string
 *                description: Session key for authentication
 *              - in: query
 *                name: password
 *                schema:
 *                    type: string
 *                description: Password for calendar access (if not using session authentication)
 *          responses:
 *              200:
 *                  description: Success
 *                  content:
 *                      application/json:
 *                          schema:
 *                              $ref: '#/components/schemas/Event'
 *              400:
 *                  description: Bad request - missing or invalid parameters
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: Please state the name of the calendar you want events from.
 *              403:
 *                  description: Forbidden - no access to the calendar
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: You do not have access to the specified calendar.
 *              404:
 *                  description: Not found - no upcoming events
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: No upcoming events found.
 *              500:
 *                  description: Server error
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  status:
 *                                      type: string
 *                                      example: PROCESSING_ERROR
 *                                  message:
 *                                      type: string
 *                                      example: Internal Server Error. Try again later.
 *                                  reference:
 *                                      type: string
 *                                      example: 6ec1361c-4175-4e81-b2ef-a0792a9a1dc3
 */
eventsRouter.get('/:calendar/json/next', async (req: Request, res: Response) => {
	try {
		// Get request params
		let calendarName: string = req.params.calendar as string ?? '';
		let sessionId: string = req.query.sessionId as string ?? '';
		let sessionKey: string = req.query.sessionKey as string ?? '';
		let password: string = req.query.password as string ?? '';
		let ip: string = req.socket.remoteAddress ?? '';

		if (calendarName.length < 1) {
			res.status(400).send({'message': 'Please state the name of the calendar you want events from.'});
			return;
		}

		if (!Array.from(calendarNames.keys()).includes(calendarName)) {
			res.status(400).send({'message': 'Unknown calendar.'});
			return;
		}

		let calendarId: number = calendarNames.get(calendarName)!.id;

		if (! await CredentialService.hasAccess(calendarName, sessionId, sessionKey, password, ip)) {
			res.status(403).send({'message': 'You do not have access to the specified calendar.'});
			return;
		}

		// Get next upcoming event
		let event = await EventService.getNextUpcomingEvent(calendarId);

		if (event === null) {
			res.status(404).send({'message': 'No upcoming events found.'});
			return;
		}

		// Send the event back
		res.status(200).send(event);
	} catch (e: any) {
		let errorGuid = Guid.create().toString();
		logger.error('Error handling a request: ' + e.message, {reference: errorGuid});
		res.status(500).send({
			'status': 'PROCESSING_ERROR',
			'message': 'Internal Server Error. Try again later.',
			'reference': errorGuid
		});
	}
});

/**
 * @swagger
 * /calendar/events/{calendar}/ical:
 *      get:
 *          summary: Get all events from a specific calendar in iCal format
 *          description: Returns all events from the specified calendar in iCal format for calendar applications. Authentication required.
 *          tags:
 *              - calendar
 *          parameters:
 *              - in: path
 *                name: calendar
 *                required: true
 *                schema:
 *                    type: string
 *                    enum: [public, members, choir, management]
 *                description: The name of the calendar to get events from
 *              - in: query
 *                name: sessionId
 *                schema:
 *                    type: string
 *                description: Session ID for authentication
 *              - in: query
 *                name: sessionKey
 *                schema:
 *                    type: string
 *                description: Session key for authentication
 *              - in: query
 *                name: password
 *                schema:
 *                    type: string
 *                description: Password for calendar access (if not using session authentication)
 *          responses:
 *              200:
 *                  description: Success - returns iCal file
 *                  content:
 *                      text/calendar:
 *                          schema:
 *                              type: string
 *                              format: binary
 *              400:
 *                  description: Bad request - missing or invalid parameters
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: Please state the name of the calendar you want events from.
 *              403:
 *                  description: Forbidden - no access to the calendar
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: You do not have access to the specified calendar.
 *              500:
 *                  description: Server error
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  status:
 *                                      type: string
 *                                      example: PROCESSING_ERROR
 *                                  message:
 *                                      type: string
 *                                      example: Internal Server Error. Try again later.
 *                                  reference:
 *                                      type: string
 *                                      example: 6ec1361c-4175-4e81-b2ef-a0792a9a1dc3
 */
eventsRouter.get('/:calendar/ical', async (req: Request, res: Response) => {
	try {
		// Get request params
		let calendarName: string = req.params.calendar as string ?? '';
		let sessionId: string = req.query.sessionId as string ?? '';
		let sessionKey: string = req.query.sessionKey as string ?? '';
		let password: string = req.query.password as string ?? '';
		let ip: string = req.socket.remoteAddress ?? '';

		if (calendarName.length < 1) {
			res.status(400).send({'message': 'Please state the name of the calendar you want events from.'});
			return;
		}

		if (!Array.from(calendarNames.keys()).includes(calendarName)) {
			res.status(400).send({'message': 'Unknown calendar.'});
			return;
		}

		let calendarId: number = calendarNames.get(calendarName)!.id;

		if (! await CredentialService.hasAccess(calendarName, sessionId, sessionKey, password, ip)) {
			res.status(403).send({'message': 'You do not have access to the specified calendar.'});
			return;
		}

		// Get events
		let events = await EventService.getAllEvents(calendarId);

		// generate file name
		let fileName = calendarNames.get(calendarName)!.name;
		let file = await iCalService.convertToIcal(events);

		// Send the ical file back
		res.set({'Content-Disposition': 'attachment; filename=' + fileName + '.ics'});
		res.status(200).send(file);
	} catch (e: any) {
		let errorGuid = Guid.create().toString();
		logger.error('Error handling a request: ' + e.message, {reference: errorGuid});
		res.status(500).send({
			'status': 'PROCESSING_ERROR',
			'message': 'Internal Server Error. Try again later.',
			'reference': errorGuid
		});
	}
});

/**
 * @swagger
 * /calendar/events:
 *      post:
 *          summary: Create a new event
 *          description: Creates a new event in the specified calendar. Authentication required.
 *          tags:
 *              - calendar
 *          parameters:
 *              - in: query
 *                name: sessionId
 *                required: true
 *                schema:
 *                    type: string
 *                description: Session ID for authentication
 *              - in: query
 *                name: sessionKey
 *                required: true
 *                schema:
 *                    type: string
 *                description: Session key for authentication
 *          requestBody:
 *              required: true
 *              content:
 *                  application/json:
 *                      schema:
 *                          type: object
 *                          required:
 *                              - calendarId
 *                              - name
 *                              - startDateTime
 *                              - endDateTime
 *                          properties:
 *                              calendarId:
 *                                  type: integer
 *                                  example: 1
 *                              name:
 *                                  type: string
 *                                  example: "Concert at Musikhochschule"
 *                              description:
 *                                  type: string
 *                                  example: "Annual concert at the Musikhochschule"
 *                              startDateTime:
 *                                  type: string
 *                                  format: date-time
 *                                  example: "2023-06-15T19:00:00.000Z"
 *                              endDateTime:
 *                                  type: string
 *                                  format: date-time
 *                                  example: "2023-06-15T21:00:00.000Z"
 *                              location:
 *                                  type: string
 *                                  example: "Musikhochschule, Karlsruhe"
 *                              url:
 *                                  type: string
 *                                  example: "https://www.nachklang.art/events/concert"
 *                              wholeDay:
 *                                  type: boolean
 *                                  example: false
 *                              status:
 *                                  type: string
 *                                  enum: [PUBLIC, PRIVATE, DRAFT, DELETED]
 *                                  example: "PUBLIC"
 *          responses:
 *              201:
 *                  description: Event created successfully
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: Event with id 123 was created successfully.
 *                                  eventId:
 *                                      type: integer
 *                                      example: 123
 *              400:
 *                  description: Bad request - missing or invalid parameters
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: Required parameters missing
 *              403:
 *                  description: Forbidden - no access to create events
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: You do not have access to the specified calendar.
 *              500:
 *                  description: Server error
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  status:
 *                                      type: string
 *                                      example: PROCESSING_ERROR
 *                                  message:
 *                                      type: string
 *                                      example: Internal Server Error. Try again later.
 *                                  reference:
 *                                      type: string
 *                                      example: 6ec1361c-4175-4e81-b2ef-a0792a9a1dc3
 */
eventsRouter.post('/', async (req: Request, res: Response) => {
	try {
		// Get params
		let sessionId: string = req.query.sessionId as string ?? '';
		let sessionKey: string = req.query.sessionKey as string ?? '';
		let ip: string = req.socket.remoteAddress ?? '';

		let user = await UserService.checkSession(sessionId, sessionKey, ip);

		if (!user.isActive) {
			res.status(403).send({'message': 'You do not have access to the specified calendar.'});
			return;
		}

		if (
			req.body.calendarId === undefined ||
			isNullOrBlank(req.body.name) ||
			req.body.startDateTime === undefined ||
			req.body.endDateTime === undefined
		) {
			res.status(400).send({'message': 'Required parameters missing'});
			return;
		}

		let event: Event = {
			eventId: -1,
			calendarId: req.body.calendarId,
			uuid: '',
			name: req.body.name,
			description: req.body.description ?? '',
			startDateTime: new Date(req.body.startDateTime),
			endDateTime: new Date(req.body.endDateTime),
			createdDate: new Date(),
			location: req.body.location ?? '',
			createdById: user.userId ?? -1,
			url: req.body.url ?? '',
			wholeDay: req.body.wholeDay ?? false,
			status: req.body.status ?? 'PUBLIC'
		};

		let eventId = await EventService.createEvent(event);

		res.status(201).send({
			'message': 'Event with id ' + eventId + ' was created successfully.',
			'eventId': eventId
		});
	} catch (e: any) {
		let errorGuid = Guid.create().toString();
		logger.error('Error handling a request: ' + e.message, {reference: errorGuid});
		res.status(500).send({
			'status': 'PROCESSING_ERROR',
			'message': 'Internal Server Error. Try again later.',
			'reference': errorGuid
		});
	}
});

/**
 * @swagger
 * /calendar/events/{eventId}:
 *      put:
 *          summary: Update an existing event
 *          description: Updates an existing event with the provided data. Authentication required.
 *          tags:
 *              - calendar
 *          parameters:
 *              - in: path
 *                name: eventId
 *                required: true
 *                schema:
 *                    type: integer
 *                description: The ID of the event to update
 *              - in: query
 *                name: sessionId
 *                required: true
 *                schema:
 *                    type: string
 *                description: Session ID for authentication
 *              - in: query
 *                name: sessionKey
 *                required: true
 *                schema:
 *                    type: string
 *                description: Session key for authentication
 *          requestBody:
 *              required: true
 *              content:
 *                  application/json:
 *                      schema:
 *                          type: object
 *                          required:
 *                              - calendarId
 *                              - name
 *                              - startDateTime
 *                              - endDateTime
 *                          properties:
 *                              calendarId:
 *                                  type: integer
 *                                  example: 1
 *                              name:
 *                                  type: string
 *                                  example: "Updated Concert at Musikhochschule"
 *                              description:
 *                                  type: string
 *                                  example: "Updated annual concert at the Musikhochschule"
 *                              startDateTime:
 *                                  type: string
 *                                  format: date-time
 *                                  example: "2023-06-15T19:00:00.000Z"
 *                              endDateTime:
 *                                  type: string
 *                                  format: date-time
 *                                  example: "2023-06-15T21:00:00.000Z"
 *                              location:
 *                                  type: string
 *                                  example: "Musikhochschule, Karlsruhe"
 *                              createdBy:
 *                                  type: string
 *                                  example: "John Doe"
 *                              url:
 *                                  type: string
 *                                  example: "https://www.nachklang.art/events/concert"
 *                              wholeDay:
 *                                  type: boolean
 *                                  example: false
 *                              status:
 *                                  type: string
 *                                  enum: [PUBLIC, PRIVATE, DRAFT, DELETED]
 *                                  example: "PUBLIC"
 *          responses:
 *              200:
 *                  description: Event updated successfully
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: Event was successfully updated
 *              400:
 *                  description: Bad request - missing or invalid parameters
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: Required parameters missing
 *              403:
 *                  description: Forbidden - no access to update events
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: You do not have access to the specified calendar.
 *              500:
 *                  description: Server error
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  status:
 *                                      type: string
 *                                      example: PROCESSING_ERROR
 *                                  message:
 *                                      type: string
 *                                      example: Internal Server Error. Try again later.
 *                                  reference:
 *                                      type: string
 *                                      example: 6ec1361c-4175-4e81-b2ef-a0792a9a1dc3
 */
eventsRouter.put('/:eventId', async (req: Request, res: Response) => {
	try {
		// Get params
		let sessionId: string = req.query.sessionId as string ?? '';
		let sessionKey: string = req.query.sessionKey as string ?? '';
		let ip: string = req.socket.remoteAddress ?? '';

		let user = await UserService.checkSession(sessionId, sessionKey, ip);

		if (!user.isActive) {
			res.status(403).send({'message': 'You do not have access to the specified calendar.'});
			return;
		}

		if (
			req.params.eventId === undefined ||
			req.body.calendarId === undefined ||
			isNullOrBlank(req.body.name) ||
			req.body.startDateTime === undefined ||
			req.body.endDateTime === undefined
		) {
			res.status(400).send({'message': 'Required parameters missing'});
			return;
		}

		let event: Event = {
			eventId: parseInt(req.params.eventId, 10),
			calendarId: req.body.calendarId,
			uuid: '',
			name: req.body.name,
			description: req.body.description ?? '',
			startDateTime: new Date(req.body.startDateTime),
			endDateTime: new Date(req.body.endDateTime),
			createdDate: new Date(),
			location: req.body.location ?? '',
			createdBy: req.body.createdBy ?? '',
			createdById: user.userId ?? -1,
			url: req.body.url ?? '',
			wholeDay: req.body.wholeDay ?? false,
			status: req.body.status ?? 'PUBLIC'
		};

		let successRows = await EventService.updateEvent(event);

		if (successRows === 1) {
			res.status(200).send({'message': 'Event was successfully updated'});
		} else if (successRows === 0) {
			res.status(200).send({'message': '0 rows were updated'});
		} else {
			res.status(500).send({'message': 'An error occurred during the update process. Please try again.'});
		}
	} catch (e: any) {
		let errorGuid = Guid.create().toString();
		logger.error('Error handling a request: ' + e.message, {reference: errorGuid});
		res.status(500).send({
			'status': 'PROCESSING_ERROR',
			'message': 'Internal Server Error. Try again later.',
			'reference': errorGuid
		});
	}
});

/**
 * @swagger
 * /calendar/events/move/{eventId}:
 *      put:
 *          summary: Move an event to a different calendar
 *          description: Moves an existing event to a different calendar. Authentication required.
 *          tags:
 *              - calendar
 *          parameters:
 *              - in: path
 *                name: eventId
 *                required: true
 *                schema:
 *                    type: integer
 *                description: The ID of the event to move
 *              - in: query
 *                name: sessionId
 *                required: true
 *                schema:
 *                    type: string
 *                description: Session ID for authentication
 *              - in: query
 *                name: sessionKey
 *                required: true
 *                schema:
 *                    type: string
 *                description: Session key for authentication
 *          requestBody:
 *              required: true
 *              content:
 *                  application/json:
 *                      schema:
 *                          type: object
 *                          required:
 *                              - calendarId
 *                          properties:
 *                              calendarId:
 *                                  type: integer
 *                                  example: 2
 *                                  description: The ID of the calendar to move the event to
 *                              name:
 *                                  type: string
 *                                  example: "Concert at Musikhochschule"
 *                              description:
 *                                  type: string
 *                                  example: "Annual concert at the Musikhochschule"
 *                              startDateTime:
 *                                  type: string
 *                                  format: date-time
 *                                  example: "2023-06-15T19:00:00.000Z"
 *                              endDateTime:
 *                                  type: string
 *                                  format: date-time
 *                                  example: "2023-06-15T21:00:00.000Z"
 *                              location:
 *                                  type: string
 *                                  example: "Musikhochschule, Karlsruhe"
 *                              createdBy:
 *                                  type: string
 *                                  example: "John Doe"
 *                              url:
 *                                  type: string
 *                                  example: "https://www.nachklang.art/events/concert"
 *                              wholeDay:
 *                                  type: boolean
 *                                  example: false
 *                              status:
 *                                  type: string
 *                                  enum: [PUBLIC, PRIVATE, DRAFT, DELETED]
 *                                  example: "PUBLIC"
 *          responses:
 *              200:
 *                  description: Event moved successfully
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: Event was successfully moved
 *              400:
 *                  description: Bad request - missing or invalid parameters
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: Required parameters missing
 *              403:
 *                  description: Forbidden - no access to move events
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: You do not have access to the specified calendar.
 *              500:
 *                  description: Server error
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  status:
 *                                      type: string
 *                                      example: PROCESSING_ERROR
 *                                  message:
 *                                      type: string
 *                                      example: Internal Server Error. Try again later.
 *                                  reference:
 *                                      type: string
 *                                      example: 6ec1361c-4175-4e81-b2ef-a0792a9a1dc3
 */
eventsRouter.put('/move/:eventId', async (req: Request, res: Response) => {
	try {
		// Get params
		let sessionId: string = req.query.sessionId as string ?? '';
		let sessionKey: string = req.query.sessionKey as string ?? '';
		let ip: string = req.socket.remoteAddress ?? '';

		let user = await UserService.checkSession(sessionId, sessionKey, ip);

		if (!user.isActive) {
			res.status(403).send({'message': 'You do not have access to the specified calendar.'});
			return;
		}

		if (
			req.params.eventId === undefined ||
			req.body.calendarId === undefined
		) {
			res.status(400).send({'message': 'Required parameters missing'});
			return;
		}

		let event: Event = {
			eventId: parseInt(req.params.eventId, 10),
			calendarId: req.body.calendarId,
			uuid: '',
			name: req.body.name,
			description: req.body.description ?? '',
			startDateTime: new Date(req.body.startDateTime),
			endDateTime: new Date(req.body.endDateTime),
			createdDate: new Date(),
			location: req.body.location ?? '',
			createdBy: req.body.createdBy ?? '',
			createdById: user.userId ?? -1,
			url: req.body.url ?? '',
			wholeDay: req.body.wholeDay ?? false,
			status: req.body.status ?? 'PUBLIC'
		};

		let success = await EventService.moveEvent(event);

		if (success) {
			res.status(200).send({'message': 'Event was successfully moved'});
		} else {
			res.status(500).send({'message': 'An error occurred during the moving process. Please try again.'});
		}
	} catch (e: any) {
		let errorGuid = Guid.create().toString();
		logger.error('Error handling a request: ' + e.message, {reference: errorGuid});
		res.status(500).send({
			'status': 'PROCESSING_ERROR',
			'message': 'Internal Server Error. Try again later.',
			'reference': errorGuid
		});
	}
});

/**
 * @swagger
 * /calendar/events/{eventId}:
 *      delete:
 *          summary: Delete an event
 *          description: Deletes an existing event. Authentication required.
 *          tags:
 *              - calendar
 *          parameters:
 *              - in: path
 *                name: eventId
 *                required: true
 *                schema:
 *                    type: integer
 *                description: The ID of the event to delete
 *              - in: query
 *                name: sessionId
 *                required: true
 *                schema:
 *                    type: string
 *                description: Session ID for authentication
 *              - in: query
 *                name: sessionKey
 *                required: true
 *                schema:
 *                    type: string
 *                description: Session key for authentication
 *          responses:
 *              200:
 *                  description: Event deleted successfully
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: Event was successfully deleted
 *              400:
 *                  description: Bad request - missing or invalid parameters
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: Required parameters missing
 *              403:
 *                  description: Forbidden - no access to delete events
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  message:
 *                                      type: string
 *                                      example: You do not have access to the specified calendar.
 *              500:
 *                  description: Server error
 *                  content:
 *                      application/json:
 *                          schema:
 *                              type: object
 *                              properties:
 *                                  status:
 *                                      type: string
 *                                      example: PROCESSING_ERROR
 *                                  message:
 *                                      type: string
 *                                      example: Internal Server Error. Try again later.
 *                                  reference:
 *                                      type: string
 *                                      example: 6ec1361c-4175-4e81-b2ef-a0792a9a1dc3
 */
eventsRouter.delete('/:eventId', async (req: Request, res: Response) => {
	try {
		// Get params
		let sessionId: string = req.query.sessionId as string ?? '';
		let sessionKey: string = req.query.sessionKey as string ?? '';
		let ip: string = req.socket.remoteAddress ?? '';

		let user = await UserService.checkSession(sessionId, sessionKey, ip);

		if (!user.isActive) {
			res.status(403).send({'message': 'You do not have access to the specified calendar.'});
			return;
		}

		if (
			req.params.eventId === undefined
		) {
			res.status(400).send({'message': 'Required parameters missing'});
			return;
		}

		let event: Event = {
			eventId: parseInt(req.params.eventId, 10),
			calendarId: -1,
			uuid: '',
			name: '',
			description: '',
			startDateTime: new Date(),
			endDateTime: new Date(),
			createdDate: new Date(),
			location: '',
			createdBy: '',
			createdById: user.userId ?? -1,
			url: '',
			wholeDay: false,
			status: 'DELETED'
		};

		let success = await EventService.deleteEvent(event);

		if (success) {
			res.status(200).send({'message': 'Event was successfully deleted'});
		} else {
			res.status(500).send({'message': 'An error occurred during deletion. Please try again.'});
		}
	} catch (e: any) {
		let errorGuid = Guid.create().toString();
		logger.error('Error handling a request: ' + e.message, {reference: errorGuid});
		res.status(500).send({
			'status': 'PROCESSING_ERROR',
			'message': 'Internal Server Error. Try again later.',
			'reference': errorGuid
		});
	}
});

/**
 * Checks if a given string is null, undefined or blank
 * @param str The string to check
 */
function isNullOrBlank(str: string | null): boolean {
	return str === null || str === undefined || str.trim() === '';
}
